@@ -1,5 +1,6 @@
 import logging
 import multiprocessing
+import sys
 import traceback
 from multiprocessing import Pool
 
@@ -31,6 +32,8 @@ class MATRIX:
     """
     object_info = None
     SDE_ROCHE = 2000
+    lcbuilder = LcBuilder()
+    MIN_SEARCH_PERIOD = 0.5
 
     def __init__(self, target, sectors, dir, preserve=False, star_info=None, file=None, exposure_time=None,
                  initial_mask=None, initial_transit_mask=None,
@@ -39,7 +42,8 @@ class MATRIX:
                  auto_detrend_enabled=False, auto_detrend_method="cosine", auto_detrend_ratio=0.25,
                  auto_detrend_period=None, prepare_algorithm=None, cache_dir=os.path.expanduser('~') + "/",
                  oscillation_reduction=False, oscillation_min_snr=4, oscillation_amplitude_threshold=0.001,
-                 oscillation_ws_percent=0.01, oscillation_min_period=0.001, cores=multiprocessing.cpu_count() - 1
+                 oscillation_ws_percent=0.01, oscillation_min_period=0.002, oscillation_max_period=0.2,
+                 cores=multiprocessing.cpu_count() - 1
                  ):
         assert target is not None and isinstance(target, str)
         assert sectors is not None and (sectors == 'all' or isinstance(sectors, list))
@@ -70,13 +74,13 @@ class MATRIX:
         self.oscillation_amplitude_threshold = oscillation_amplitude_threshold
         self.oscillation_ws_percent = oscillation_ws_percent
         self.oscillation_min_period = oscillation_min_period
+        self.oscillation_max_period = oscillation_max_period
         self.prepare_algorithm = prepare_algorithm
         self.cache_dir = cache_dir
         self.cores = cores
 
     def retrieve_object_data(self, inject_dir=None):
-        lcbuilder = LcBuilder()
-        self.object_info = lcbuilder.build_object_info(self.id, None, self.sectors, self.file, self.exposure_time,
+        self.object_info = self.lcbuilder.build_object_info(self.id, None, self.sectors, self.file, self.exposure_time,
                                                        None, None,
                                                        self.star_info, None,
                                                        self.eleanor_corr_flux, self.outliers_sigma,
@@ -86,10 +90,11 @@ class MATRIX:
                                                        self.auto_detrend_ratio, self.auto_detrend_period,
                                                        self.prepare_algorithm, False,
                                                        self.oscillation_min_snr, self.oscillation_amplitude_threshold,
-                                                       self.oscillation_ws_percent, self.oscillation_min_period)
+                                                       self.oscillation_ws_percent, self.oscillation_min_period,
+                                                       self.oscillation_max_period)
         if inject_dir is None:
             inject_dir = self.build_inject_dir()
-        self.lc_build = lcbuilder.build(self.object_info, inject_dir, self.cache_dir)
+        self.lc_build = self.lcbuilder.build(self.object_info, inject_dir, self.cache_dir)
         if self.star_info is None:
             self.star_info = self.lc_build.star_info
         self.ab = self.star_info.ld_coefficients
@@ -109,8 +114,8 @@ class MATRIX:
         return inject_dir
 
     def retrieve_object_data_for_recovery(self, inject_dir, recovery_file):
-        lcbuilder = LcBuilder()
-        self.object_info = lcbuilder.build_object_info(self.id, None, None, recovery_file, self.exposure_time,
+        self.__setup_logging(inject_dir)
+        self.object_info = self.lcbuilder.build_object_info(self.id, None, None, recovery_file, self.exposure_time,
                                                        self.initial_mask, self.initial_transit_mask,
                                                        self.star_info, None,
                                                        self.eleanor_corr_flux, self.outliers_sigma,
@@ -120,23 +125,42 @@ class MATRIX:
                                                        self.auto_detrend_ratio, self.auto_detrend_period,
                                                        self.prepare_algorithm, self.oscillation_reduction,
                                                        self.oscillation_min_snr, self.oscillation_amplitude_threshold,
-                                                       self.oscillation_ws_percent, self.oscillation_min_period)
-        self.lc_build = lcbuilder.build(self.object_info, inject_dir, self.cache_dir)
+                                                       self.oscillation_ws_percent, self.oscillation_min_period,
+                                                       self.oscillation_max_period)
+
+        if self.object_info.reduce_simple_oscillations and \
+                self.object_info.oscillation_max_period < self.object_info.oscillation_min_period:
+            logging.info("Stellar oscillation period has been set to empty. Defaulting to 1/3 the minimum search period")
+            self.object_info.oscillation_max_period = self.MIN_SEARCH_PERIOD / 3
+        self.lc_build = self.lcbuilder.build(self.object_info, inject_dir, self.cache_dir)
 
     def build_inject_dir(self):
         inject_dir = self.dir + "/" + self.object_info.mission_id().replace(" ", "") + "_ir/"
-        if os.path.isdir(inject_dir):
-            folder_list = sorted([x[0] for x in os.walk(self.dir) if x[0].find(self.object_info.mission_id().replace(" ", "")) > 0], key=os.path.getmtime)
-            last_element = folder_list[len(folder_list)-1]
-            last_number_index = last_element.rfind("_")
-            number = last_element[last_number_index+1:len(last_element)]
-            if number.isdigit():
-                inject_dir = self.dir + "/" + inject_dir[:-1] + "_" + str(int(number) + 1) + "/"
-            else:
-                inject_dir = self.dir + "/" + inject_dir[:-1] + "_1/"
+        index = 0
+        while os.path.exists(inject_dir) or os.path.isdir(inject_dir):
+            inject_dir = self.dir + "/" + self.object_info.mission_id().replace(" ", "") + "_ir_" + str(index) + "/"
+            index = index + 1
         os.mkdir(inject_dir)
+        self.__setup_logging(inject_dir)
         return inject_dir
 
+    def __setup_logging(self, inject_dir):
+        file_dir = inject_dir + "matrix.log"
+        formatter = logging.Formatter('%(message)s')
+        logger = logging.getLogger()
+        while len(logger.handlers) > 0:
+            logger.handlers.pop()
+        logger.setLevel(logging.INFO)
+        handler = logging.StreamHandler(sys.stdout)
+        handler.setLevel(logging.INFO)
+        handler.setFormatter(formatter)
+        logger.addHandler(handler)
+        handler = logging.FileHandler(file_dir)
+        handler.setLevel(logging.INFO)
+        handler.setFormatter(formatter)
+        logger.addHandler(handler)
+        logging.info("Setup injection directory")
+
     def inject(self, phases, min_period, max_period, steps_period, min_radius, max_radius, steps_radius,
                period_grid_geom="lin", radius_grid_geom="lin"):
         assert phases is not None and isinstance(phases, int) and phases > 0
@@ -180,7 +204,7 @@ class MATRIX:
         reports_df = pd.DataFrame(columns=['period', 'radius', 'epoch', 'duration_found', 'period_found', 'epoch_found',
                                            'found', 'snr', 'sde', 'run'])
         for file in sorted(os.listdir(inject_dir)):
-            file_name_matches = re.search("P([0-9]+\\.[0-9])+_R([0-9]+\\.[0-9]+)_([0-9]+\\.[0-9]+)\\.csv", file)
+            file_name_matches = re.search("P([0-9]+\\.[0-9]+)+_R([0-9]+\\.[0-9]+)_([0-9]+\\.[0-9]+)\\.csv", file)
             if file_name_matches is not None:
                 try:
                     period = float(file_name_matches[1])
@@ -197,13 +221,11 @@ class MATRIX:
                         epoch_found = 0
                         period_found = 0
                     else:
-                        self.retrieve_object_data_for_recovery(inject_dir, inject_dir + file)
-                        flux = self.lc_build.lc.flux.value
-                        time = self.lc_build.lc.time.value
+                        self.retrieve_object_data_for_recovery(inject_dir + "/", inject_dir + file)
                         found, snr, sde, run, duration_found, period_found, epoch_found = \
-                            self.__search(time, flux, self.radius, self.radiusmin,
+                            self.__search(self.lc_build.lc.time.value, self.lc_build.lc.flux.value, self.radius, self.radiusmin,
                                           self.radiusmax, self.mass, self.massmin,
-                                          self.massmax, self.ab, epoch, period, 0.5,
+                                          self.massmax, self.ab, epoch, period, self.MIN_SEARCH_PERIOD,
                                           max_period_search, snr_threshold,
                                           transit_template, detrend_ws, self.lc_build.transits_min_count,
                                           run_limit, custom_search_algorithm)
@@ -429,30 +451,68 @@ class MATRIX:
         plt.savefig(inject_dir + '/inj-rec.png', bbox_inches='tight', dpi=200)
         plt.close()
 
-        # period_grid = np.linspace(min_period, max_period, len(df["period"].unique()))
-        # radius_grid = np.round(np.linspace(min_rad, max_rad, len(df["radius"].unique())), 2)
-        # bins = [period_grid, radius_grid]
-        # h1, x, y = np.histogram2d(df['period'][df['found'] == 1], df['radius'][df['found'] == 1], bins=bins)
-        # h2, x, y = np.histogram2d(df['period'][df['found'] == 0], df['radius'][df['found'] == 0], bins=bins)
-        # normed_hist = (100. * h1 / (h1 + h2))
-        # fig, ax = plt.subplots(figsize=(2.7 * 5, 5))
-        # im = plt.imshow(normed_hist.T, origin='lower', extent=(x[0], x[-1], y[0], y[-1]), interpolation='none',
-        #                 aspect='auto', cmap='viridis', vmin=0, vmax=100, rasterized=True)
-        # plt.colorbar(im, label='Recovery rate (%)')
-        # plt.xlabel('Injected period (days)')
-        # plt.ylabel(r'Injected radius (R$_\oplus$)')
-        # ax.set_title(object_id + " - P/R recovery (" + str(phases) + " " + phases_str + ")")
-        # if xticks is not None:
-        #     plt.xticks(xticks)
-        # else:
-        #     period_ticks_decimals = MATRIX.num_of_zeros(max_period - min_period) + 1
-        #     plot_bins = 10 if 10 < len(period_grid) else len(period_grid)
-        #     plt.locator_params(axis="x", nbins=plot_bins)
-        #     ax.xaxis.set_major_formatter(FormatStrFormatter('%.' + str(period_ticks_decimals) + 'f'))
-        # if yticks is not None:
-        #     plt.xticks(yticks)
-        # plt.savefig(inject_dir + '/inj-rec.png', bbox_inches='tight', dpi=200)
-        # plt.close()
+    @staticmethod
+    def plot_diff(object_id, inject_dir1, inject_dir2, output_dir, binning=1, xticks=None, yticks=None,
+                  period_grid_geom="lin", radius_grid_geom="lin"):
+        df1 = pd.read_csv(inject_dir1 + '/a_tls_report.csv', float_precision='round_trip', sep=',',
+                         usecols=['period', 'radius', 'found', 'sde'])
+        df2 = pd.read_csv(inject_dir2 + '/a_tls_report.csv', float_precision='round_trip', sep=',',
+                         usecols=['period', 'radius', 'found', 'sde'])
+        min_period = df1["period"].min()
+        max_period = df1["period"].max()
+        min_rad = df1["radius"].min()
+        max_rad = df1["radius"].max()
+        phases = len(df1[df1["period"] == df1["period"].min()][df1["radius"] == df1["radius"].min()])
+        phases_str = "phase" if phases == 1 else "phases"
+        bin_nums = int(np.ceil(len(df1["period"].unique()) / binning))
+        if period_grid_geom == 'lin':
+            step_period = (max_period - min_period) / (len(df1["period"].unique()) - 1)
+            step_period = step_period * binning
+            if step_period <= 0:
+                step_period = 0.1
+            period_grid = np.linspace(min_period, max_period, bin_nums)\
+                if max_period - min_period > 0 else np.full((1), min_period)
+        else:
+            period_grid = np.logspace(np.log10(min_period), np.log10(max_period), bin_nums)
+        bin_nums = int(np.ceil(len(df1["radius"].unique()) / binning))
+        if radius_grid_geom == 'lin':
+            step_radius = (max_rad - min_rad) / (len(df1["radius"].unique()) - 1)
+            step_radius = step_radius * binning
+            if step_radius <= 0:
+                step_radius = 0.1
+            radius_grid = np.round(np.linspace(min_rad, max_rad, bin_nums), 2)\
+                if max_rad - min_rad > 0 else np.full((1), min_rad)
+        else:
+            radius_grid = np.round(np.logspace(np.log10(min_rad), np.log10(max_rad), 2), bin_nums)
+        f = len(period_grid) / len(radius_grid)
+        bins = [period_grid, radius_grid]
+        h11, x1, y1 = np.histogram2d(df1['period'][df1['found'] == 1], df1['radius'][df1['found'] == 1], bins=bins)
+        h12, x1, y1 = np.histogram2d(df1['period'][df1['found'] == 0], df1['radius'][df1['found'] == 0], bins=bins)
+        h21, x2, y2 = np.histogram2d(df2['period'][df2['found'] == 1], df2['radius'][df2['found'] == 1], bins=bins)
+        h22, x2, y2 = np.histogram2d(df2['period'][df2['found'] == 0], df2['radius'][df2['found'] == 0], bins=bins)
+        h1 = h11 - h12
+        h2 = h21 - h22
+        normed_hist1 = phases * h11 / (h11 + h12)
+        normed_hist2 = phases * h21 / (h21 + h22)
+        normed_hist = normed_hist1 - normed_hist2
+        fig, ax = plt.subplots(figsize=(2.7 * 5, 5))
+        im = plt.imshow(normed_hist.T, origin='lower', extent=(x1[0], x1[-1], y1[0], y1[-1]), interpolation='none',
+                        aspect='auto', cmap='viridis', vmin=-phases, vmax=phases, rasterized=True)
+        plt.colorbar(im, label='# Found samples diff.')
+        plt.xlabel('Injected period (days)')
+        plt.ylabel(r'Injected radius (R$_\oplus$)')
+        ax.set_title(object_id + " - P/R recovery diff(" + str(phases) + " " + phases_str + ")")
+        if xticks is not None:
+            plt.xticks(xticks)
+        else:
+            period_ticks_decimals = MATRIX.num_of_zeros(max_period - min_period) + 1
+            plot_bins = 10 if 10 < len(period_grid) else len(period_grid)
+            plt.locator_params(axis="x", nbins=plot_bins)
+            ax.xaxis.set_major_formatter(FormatStrFormatter('%.' + str(period_ticks_decimals) + 'f'))
+        if yticks is not None:
+            plt.xticks(yticks)
+        plt.savefig(output_dir + '/inj-rec-diff.png', bbox_inches='tight', dpi=200)
+        plt.close()
 
     def __search(self, time, flux, rstar, rstar_min, rstar_max, mass, mstar_min, mstar_max, ab, epoch,
                  period, min_period, max_period, min_snr, transit_template, ws, transits_min_count,
